/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*///*****************************************************************************

#ifndef PACKET_H_
#define PACKET_H_

#include "defines.h"
#include "FECEncoder.h"

/****************************************************************************************
     5B       5B         1B        4B      1B        2B     0-492B      1B
  | Sync | Preamble | Delimiter | Addr | Command | Length |  Data   | Checksum |
*****************************************************************************************/
typedef struct _Packet //protocol packet format
{
	uint_8 delimiter; 		// delimiter field
	uint_32 addr; 		// address field, 5 byte
	uint_8 command;			// command field
	uint_16 dataLength;	// data length field
	uint_8 *data;           // data field
} Packet;

//this function format the packet. its parameter is a reference of variable packet
//and its return value is the check sum value of the packet, using XOR operation
uint_8 formatPacket(Packet *);//return check sum value

//this function sends the packet to send buffer, with the right order
void sendPacket (Packet *, uint_16 *, uint_8);

#endif /* PACKET_H_ */
