/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*///*****************************************************************************


#include "System.h"

void systemClockInit(void)
{
	CS_setExternalClockSourceFrequency(32768 , 48000000);//LFXT = 32.768kHz, HFXT = 48MHz
	GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_PJ ,
				GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 | GPIO_PIN3 ,
				GPIO_PRIMARY_MODULE_FUNCTION); //PJ.0-PJ.3 used as XT

	//Setting for 48MHz CPU clock
	PCM_setCoreVoltageLevel(PCM_VCORE1);
	FlashCtl_setWaitState(FLASH_BANK0, 2);
	FlashCtl_setWaitState(FLASH_BANK1, 2);
	CS_startHFXT(false);

	CS_initClockSignal(CS_ACLK , CS_LFXTCLK_SELECT , CS_CLOCK_DIVIDER_1);		//ACLK = LFXT 32kHz
	CS_initClockSignal(CS_MCLK , CS_HFXTCLK_SELECT , CS_CLOCK_DIVIDER_1);		//MCLK = HFXT 48MHz
	CS_initClockSignal(CS_HSMCLK , CS_HFXTCLK_SELECT , CS_CLOCK_DIVIDER_1);		//HSMCLK = HFXT 48MHz
	CS_initClockSignal(CS_SMCLK, CS_HFXTCLK_SELECT, CS_CLOCK_DIVIDER_32);		//SMCLK =  HFXT/32 1.5MHz
}

void timerA0Init(void)//use TA0 as carrier, carrier frequency is 125kHz
{
	GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P2, GPIO_PIN4 | GPIO_PIN5,
	            GPIO_PRIMARY_MODULE_FUNCTION);
	TA0CTL = TASSEL__SMCLK | ID__2 | TACLR;
	TA0CCR0 = 5;
	TA0CCR1 = 3;//channel 1 is just for monitor
	TA0CCR2 = 3;
	TA0CCTL1 = OUTMOD_2;
	TA0CCTL2 = OUTMOD_6;
}

void timerA1Init(void)//use TA1 as bit rate generator
{
	TA1CTL = TASSEL__SMCLK | ID__2 | TACLR;
	TA1CCTL0 = CCIE;
	Interrupt_enableInterrupt(INT_TA1_0);
}

void timerA2Init(void)//use TA2 for cpu calculate
{
	TA2CTL = TASSEL__SMCLK | TACLR;
	TA2CCTL0 = CCIE;
	TA2CCR0 = 0xb5d;
	Interrupt_enableInterrupt(INT_TA2_0);
}

void initP1ISR(void)
{
	P1OUT |= BIT1;
	P1DIR &= ~BIT1;
	P1REN |= BIT1;
	P1IE |= BIT1;
	P1IES |= BIT1;
	P1IFG &= ~BIT1;
	Interrupt_enableInterrupt(INT_PORT1);
}
