/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*///*****************************************************************************


#include "defines.h"
#include "Packet.h"
#include "System.h"
#include "stdio.h"
#include "stdlib.h"
#include "Modulation.h"

uint_8 forDebug = 0;
Send_Info instSendInfo =
{
	.length = 0,
	.counter = 0,
	.flag = 0
};

Packet sendTestData =
{
	.delimiter = 0b10000010,
	.addr = 0x12345678,
	.command = 1,
	.dataLength = 191
};

uint_8 sendBitFlag = 0;
volatile uint_8 intFlag = 0;
uint_32 cpuCounter = 0;

void main(void)
{
    WDT_A_holdTimer();           // Stop watchdog timer
    uint_16 i;
    systemClockInit();
    initP1ISR();
    uint_8 sendData[200];
    for(i = 0;i<200;i++)
    	sendData[i] = i;

    sendTestData.data = &sendData[0];

    uint_16 buffer[530];
    uint_16 *sendBuffer;

    timerA0Init();
    timerA1Init();
    setBitRate(BIT_RATE_125000);

    sendBuffer = buffer;

    initSendInfo(&instSendInfo , ENABLE_FEC);
    sendBitFlag = 0;

    sendPacket(&sendTestData , sendBuffer , ENABLE_FEC);

    uint_16 packetLength = (*sendBuffer) & 0x00ff;
    sendBuffer++;
    packetLength = (packetLength << 8) | ((*sendBuffer) & 0x00ff);

    sendBuffer++;
    sendBitFlag = 1;
    packetLength = 0;

    //TA2CTL |= MC__UP;

	Interrupt_enableMaster();
	cpuCounter = 0;

    while(1)//
    {
    	if(intFlag)
    	{
    		intFlag = 0;
    		GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P2, GPIO_PIN5 | GPIO_PIN4, GPIO_PRIMARY_MODULE_FUNCTION);//enable carrier output
    		startCarrier();
			startSignal();
    	}

		if(sendBitFlag)//send interrupt flag
		{
			sendBitFlag = 0;
			if(packetLength != 0)
			{
				if(instSendInfo.counter == instSendInfo.length)
				{
					sendBuffer++;
					instSendInfo.counter = 0;
					packetLength--;
				}
				modulateOneBit(*sendBuffer);//modulation
				*sendBuffer = *sendBuffer >> 1;
				instSendInfo.counter++;
			//send entry
			}
			else
			{//modulation stop
				stopSignal();
				stopCarrier();
				TA2CTL = TASSEL__SMCLK;
				TA2CTL |= TACLR;
				P1IE |= BIT1;
				GPIO_setAsOutputPin(GPIO_PORT_P2 , GPIO_PIN5);
				GPIO_setOutputLowOnPin(GPIO_PORT_P2 , GPIO_PIN5);
				sendTestData.data = &sendData[0];
				sendBuffer = buffer;
				sendPacket(&sendTestData , sendBuffer , ENABLE_FEC);
				packetLength = (*sendBuffer)  & 0x00ff;
				sendBuffer++;
				packetLength = (packetLength << 8) | ((*sendBuffer) & 0x00ff);
				sendBuffer++;
			}
		}
    	else
    		cpuCounter++;
    }
}

void TA1_0_IRQHandler(void)
{
	TA1CCTL0 &= ~CCIFG;
	sendBitFlag = 1;
}

void TA2_0_IRQHandler(void)
{
	TA2CCTL0 &= ~CCIFG;
	cpuCounter = 0;
}

void PORT1_IRQHandler(void)
{
	intFlag = 1;
	P1IFG &= ~BIT1;
	P1IE &= ~BIT1;
	//cpuCounter = 0;
	//TA2CTL |= MC__UP;
}
